# 将单个组件的版本号更新到podfile
from iosci.Podfile.CIComponentModel import CIComponentModel
from iosci.xcode.CIPodfileParse import CIPodfileParse

# 这里的version内容可以是版本号 也可能是 git地址
def update_component_to_model(name, version, branch, podfile_model):
    # 组件在podfile在是否存在
    exists = False
    # 找出第一个target
    first_target = []
    # 组件在podfile中存在，更新版本号
    for target in podfile_model:
        if isinstance(target, list):
            if len(first_target) == 0:
                first_target = target

            for component in target:
                if isinstance(component, CIComponentModel):
                    podfile_component = component.name.lstrip().split('/')[0]
                    target_component = 'pod %s' % name
                    if target_component.replace('\'','') == podfile_component.replace('\'',''):
                        if 'https://' in version or 'http://' in version:
                            component.git = f":git => {version}"
                            component.branch = f":branch => '{branch}'"
                        else:
                            component.git = ''
                            component.version = version
                        exists = True
                        continue
    # 组件在podfile中不存在，添加组件到第一个target中
    if not exists:
        if name and version:
            cm = CIComponentModel()
            cm.name = '  pod \'%s\'' % name
            if 'https://' in version or 'http://' in version:
                cm.git = f":git => {version}"
                cm.branch = f":branch => '{branch}'"
            else:
                cm.git = ''
                cm.version = version
            first_target.insert(1, cm)
        else:
            raise Exception("名称%s 或者版本号%s 不存在" % (name, version))


# 将最终结果重写到podfile中
def rewrite_to_podfile(podfile_path, podfile_model):
    contents = ''
    # model 转为字符串
    for target in podfile_model:
        if isinstance(target, list):
            for component in target:
                if isinstance(component, CIComponentModel):
                    # 有git地址优先用git地址，有版本号直接用版本号，没版本号则原样拼出
                    if component.git:
                        if component.configurations:
                            if component.branch:
                                pod = "%s, %s , %s,  %s\n" % (component.name, component.git, component.branch, component.configurations)
                            else:
                                pod = "%s, %s , %s\n" % (component.name, component.git, component.configurations)
                        else:
                            if component.branch:
                                pod = "%s, %s, %s \n" % (component.name, component.git, component.branch)
                            else:
                                pod = "%s, %s \n" % (component.name, component.git)

                    elif component.version:
                        # configurations 的配置，在什么情况下都应该保留
                        if component.configurations:
                            pod = "%s, %s , %s\n" % (component.name, component.version, component.configurations)
                        else:
                            pod = "%s, %s \n" % (component.name, component.version)
                    else:
                        pod = component.name
                        if component.git:
                            pod += ", %s" % component.git
                        if component.branch:
                            pod += ", %s" % component.branch
                        if component.other:
                            pod += ", %s" % component.other
                        # configurations 的配置，在什么情况下都应该保留
                        if component.configurations:
                            pod += ", %s" % component.configurations
                    contents += pod
                else:
                    contents += component
        else:
            contents += target
    with open(podfile_path, 'w+') as podfile:
        podfile.write(contents)


# 更新podfile中的组件指向新版本号
# 传过来的参数是：[组件名:版本号]
def update_component_with_newversion(podfile_path, components, branch):
    print('获取到参数为：', podfile_path, components)
    # 读取podfile文件，翻译为对象
    podfile_model = CIPodfileParse.getPodfileModel(podfile_path)
    # 把传过来的组件更新到对象中
    for component in components:
        if 'https://' in component or 'http://' in component:
            _componentSeg = component.split(':', 1)
        else:
            _componentSeg = component.split(':')
        if len(_componentSeg) == 2:
            name = _componentSeg[0]
            version = _componentSeg[1]
            update_component_to_model(name, "'%s'" % version, branch, podfile_model)
        else:
            raise Exception("传过来的组件格式不对 %s" % component)

    # 把新的内容写入到podfile中
    rewrite_to_podfile(podfile_path, podfile_model)


if __name__ == '__main__':
    f_podfile = '/Users/lch/Desktop/Podfile'
    branch = 'feature_1011_ziru'
    update_component_with_newversion(f_podfile, ['aaa:https://gitlab.xxx.com/a.git'], branch)




import os


def os_system(cmd):
    try:
        n = os.system(cmd) >> 8
        if n != 0:
            print('shell执行失败：%d %s' % (n, cmd))
            raise Exception(cmd)
    except Exception as e:
        raise e


def os_popen(cmd):
    try:
        cmd = (cmd + '; if [ $? -ne 0 ]; then echo "{{error}}"; fi').replace(';;',';')
        with os.popen(cmd) as p:
            output = p.read()
            if '{{error}}' in output:
                output = output.replace('{{error}}','')
                cmd = cmd.replace('if [ $? -ne 0 ]; then echo "{{error}}"; fi', '')
                err = '命令执行失败：%s output=%s' % (cmd, output)
                print(err)
                return (False, err)
            else:
                return (True, 'success')
    except Exception as e:
        raise e


#  ShellCMD.py 执行shell脚本的工具类
if __name__ == '__main__':
    # os_system('cat aaa')
    (status, msg) = os_popen('xcodebuild  aaa > /dev/null')
    print(status)
    print(msg)




'''
版本号管理
'''

import re


class CIVersionManager(object):

    '''
    判断传入的字符串是否符合版本号结构
    '''
    @staticmethod
    def isVersion(version):
        rule = "\.{2,10}"
        rule2 = "^\d+(\.\d+){0,10}$"
        res = re.search(rule, version)
        if (res == None):
            result = re.search(rule2, version)
            if (result):
                return True
            else:
                return False
        else:
            return False

    # version1----第一个要比较的版本字符串
    # version2----第二个要比较的版本字符串
    # split_flag----版本分隔符，默认为"."，可自定义
    # 返回值----相等返回0，version1比version2大返回1，version2比version1大返回2
    # 接受的版本字符形式----空/x/x.y/x.y./x.y.z；两个参数可为前边列出的形式的任一种
    @staticmethod
    def compare_version(version1=None, version2=None, split_flag="."):
        # 如果存在有为空的情况则进入
        if (version1 is None) or (version1 == "") or (version2 is None) or (version2 == ""):
            # version1为空且version2不为空，则返回version2大
            if ((version1 is None) or (version1 == "")) and (version2 is not None) and (version2 != ""):
                return 2
            # version2为空且version1不为空，则返回version1大
            if ((version2 is None) or (version2 == "")) and (version1 is not None) and (version1 != ""):
                return 1

        # 如果版本字符串相等，那么直接返回相等，这句会且只会在第一次比较时才可能进入
        # version1和version2都为空时也会进入这里
        if version1 == version2:
            return 0

        # 对版本字符串从左向右查找"."，第一个"."之前的字符串即为此次要比较的版本
        # 如1.3.5中的1
        try:
            current_section_version1 = version1[:version1.index(split_flag)]
        except:
            current_section_version1 = version1
        try:
            current_section_version2 = version2[:version2.index(split_flag)]
        except:
            current_section_version2 = version2
        # 对本次要比较的版本字符转成整型进行比较
        if int(current_section_version1) > int(current_section_version2):
            return 1
        elif int(current_section_version1) < int(current_section_version2):
            return 2

        # 如果本次传来版本字符串中已没有版本号分隔符，那说明本次比较的版本号已是最后一位版本号，下次比较值赋空
        # 如本次传来的是5，那下次要比较的只能赋空
        try:
            other_section_version1 = version1[version1.index(split_flag) + 1:]
        except:
            other_section_version1 = ""
        try:
            other_section_version2 = version2[version2.index(split_flag) + 1:]
        except:
            other_section_version2 = ""
        # 递归调用比较
        return CIVersionManager.compare_version(other_section_version1, other_section_version2)

    # 此函数调用compare_version()，打印比较结果
    @staticmethod
    def pick_up_latest_version(version1, version2):
        flag = CIVersionManager.compare_version(version1, version2)
        if flag == 0:
            print(f"version1 = {version1}, version2 = {version2}, the two version is equal")
        elif flag == 1:
            print(f"version1 = {version1}, version2 = {version2}, the latest version is version1 {version1}")
        elif flag == 2:
            print(f"version1 = {version1}, version2 = {version2}, the latest version is version2 {version2}")


if __name__ == '__main__':
    isversion = CIVersionManager.isVersion('1.1.2')
    print(isversion)
    CIVersionManager.pick_up_latest_version('1.1.2','1.1.11')


'''

解析podfile到model对象

podfile有特殊要求：
target end必须成对出现，不支持嵌套

'''
from iosci.Podfile.CIComponentModel import CIComponentModel
from iosci.Util.CIVersionManager import CIVersionManager


class CIPodfileParse(object):

    # 检查podfile是否符合规范，不支持嵌套target
    def checkForPodfile(podfile_lines):
        targetBegin = 0
        for line in podfile_lines:
            podfile_lstrip_line = line.lstrip()
            if podfile_lstrip_line.startswith("target "):
                targetBegin += 1
                if targetBegin >= 2:
                    return False
            if podfile_lstrip_line.startswith("end"):
                targetBegin -= 1
        return True

    '''
    解析podfile中的pod每一行到ComponentModel对象
    '''
    @staticmethod
    def parseForPodLine(line):
        pod_strip_line_segs = line.replace(" ", "").split(',')
        cm = CIComponentModel()
        for segment in pod_strip_line_segs:
            if 'pod\'' in segment or 'pod\"' in segment:
                cm.name = segment.replace('pod\'', '  pod \'').replace('pod\"', '  pod \"').replace('\"',"\'")
            elif ':git' in segment:
                cm.git = segment
            elif ':branch' in segment:
                cm.branch = segment
            elif ':configurations' in segment:
                cm.configurations = segment
            elif CIVersionManager.isVersion(segment.replace('\'', '').replace('\"', '')):
                cm.version = segment
            else:
                cm.other = segment
        return cm

    '''
    解析podfile中每一行成model对象
    '''
    @staticmethod
    def parseForPodfile(podfile_lines):
        # 是否到target行
        targetBegin = False
        # 是否target下对应的end出来
        outTarget = True
        ret = []
        for podfile_line in podfile_lines:
            podfile_lstrip_line = podfile_line.lstrip()
            # 这些开头的字段，左边如果有空格就全去掉
            if podfile_lstrip_line.startswith("install! ") or \
                    podfile_lstrip_line.startswith("source ") or \
                    podfile_lstrip_line.startswith("target ") or \
                    podfile_lstrip_line.startswith("pod ") or \
                    podfile_lstrip_line.startswith("platform "):
                pass
            elif not podfile_lstrip_line:
                continue
            else:
                podfile_lstrip_line = podfile_line

            if podfile_lstrip_line.startswith("target "):
                targetBegin = True
                outTarget = False
                targetArr = []
            elif podfile_lstrip_line.startswith("end"):
                if targetBegin:
                    targetBegin = False
                    targetArr.append(podfile_lstrip_line + '\n')
            elif not podfile_lstrip_line:  # 空行处理
                continue

            # 已经进入target体，开始解析target内容
            if targetBegin:
                if podfile_lstrip_line.startswith("pod "):
                    cm = CIPodfileParse.parseForPodLine(podfile_lstrip_line)
                    contained = False
                    for pod in targetArr:
                        if isinstance(pod, CIComponentModel) and pod.name == cm.name:
                            contained = True
                            break
                    if not contained:
                        targetArr.append(cm)
                else:
                    targetArr.append(podfile_lstrip_line)
            elif not outTarget:
                outTarget = True
                ret.append(targetArr)
            else:
                ret.append(podfile_lstrip_line)
        return ret

    '''
    传入Podfile文件路径，获取到podfile Model对象
    '''
    @staticmethod
    def getPodfileModel(podfile_path):
        with open(podfile_path, 'r') as podfile:
            _podfile_lines = podfile.readlines()
            if not CIPodfileParse.checkForPodfile(_podfile_lines):
                print("podfile 不符合要求（不能target嵌套）")
                exit(-90)
            podfile_model = CIPodfileParse.parseForPodfile(_podfile_lines)
            return podfile_model
        raise Exception("读取podfile失败")


if __name__ == '__main__':
    # podfile = '/Users/lch/Documents/ziroom/proj/ziroom-client-ios/Podfile'
    podfile = '/Users/lch/Desktop/Podfile'

    podfile_model = CIPodfileParse.getPodfileModel(podfile)
    print(podfile_model)


'''
Podfile 中的组件转为对象Model
'''


class CIComponentModel(object):
    # 组件名
    name = ''
    # 版本号
    version = ''
    # git地址
    git = ''
    # 分支名
    branch = ''
    # configurations
    configurations = ''
    # other
    other = ''

    def __init__(self, name='', version='', git='', branch='', configurations='', other=''):
        self.name = name
        self.version = version
        self.git = git
        self.branch = branch
        self.configurations = configurations
        self.other = other

    def logInfo(self):
        print('---------------------------------------------')
        print("name:", self.name)
        print("version:", self.version)
        print("git:", self.git)
        print("branch:", self.branch)
        print("configurations:", self.configurations)
        print("other:", self.other)

if __name__ == '__main__':
    model = CIComponentModel(name='YYModel', version='1.1.0', git='https://gitlab.com/YYModel.git', configurations='configurations')
    model.logInfo()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

import setuptools

version = "0.1.2.7"
with open(os.path.join(os.path.dirname(__file__), 'README.md'), encoding='utf-8') as readme:
    README = readme.read()
setuptools.setup(
    name="iosci",
    version=version,
    author="lichanghong",
    author_email="1211054926@qq.com",
    description="T解析podfile和批量修改组件版本号.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://hehuoya.com",
    install_requires=[
        'GitPython'
    ],
    packages=setuptools.find_packages(exclude=("iosci")),
    classifiers=(
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5"
    ),
    exclude_package_data={'': ["iosci/test.py", "iosci/xcode/a.py","iosci/xcode/b.py"]}
)